@@ -13,7 +13,7 @@
 #'   The function obtains the multiplier effects, on the explained variable, 
 #'   of a change in a regressor for the model that has been estimated. 
 #'   For reasons given below, this function only applies to models with an 
-#'   autoregressive structure ("slm", "sdm" and "sarar") or with spatial lags 
+#'   autoregressive structure ("slm", "sdm", "sarar" and "gnm") or with spatial lags 
 #'   of the regressors ("slx", "sdem"). \cr
 #'   The measurement of the multiplier effects is a bit more complicated than 
 #'   in a pure time series context because, due to the spatial structure of 
@@ -155,112 +155,126 @@
 #' \code{\link{spsur3sls}}
 #'
 #' @examples
-#' ## VIP: If you want to examine a particular example eliminate '#' and 
-#' ## execute the code of the example (they have been commented to 
-#' ## pass the checking time in CRAN)
 #' 
 #' ## VIP: The output of the whole set of the examples can be examined 
 #' ## by executing demo(demo_impactspsur, package="spsur")
 #' 
-#' 
+#' \donttest{
 #' ###############################################
 #' ### PURE CROSS SECTIONAL DATA(G>1; Tm=1) ######
 #' ###############################################
 #'
 #' #### Example 1: Spatial Phillips-Curve. Anselin (1988, p. 203)
-#' # rm(list = ls()) # Clean memory
-#' # data(spc)
-#' # lwspc <- spdep::mat2listw(Wspc, style = "W")
-#' # Tformula <- WAGE83 | WAGE81 ~ UN83 + NMR83 + SMSA | UN80 + NMR80 + SMSA
+#'  rm(list = ls()) # Clean memory
+#'  data(spc)
+#'  lwspc <- spdep::mat2listw(Wspc, style = "W")
+#'  Tformula <- WAGE83 | WAGE81 ~ UN83 + NMR83 + SMSA | UN80 + NMR80 + SMSA
 #' ## For SLM, SDM and SARAR models the output is a list of "lagImpact" objects
 #' ## See spatialreg::impacts for details.
-#' # spcsur_slm <-spsurml(formula = Tformula, data = spc, 
-#' #                      type = "slm", listw = lwspc)
-#' # summary(spcsur_slm)
-#' # impacts_slm <- impactspsur(spcsur_slm, listw = lwspc, R = 1000)
+#'  spcsur_slm <-spsurml(formula = Tformula, data = spc, 
+#'                       type = "slm", listw = lwspc)
+#'  summary(spcsur_slm)
+#'  impacts_slm <- impactspsur(spcsur_slm, listw = lwspc, R = 1000)
 #' ## Impacts equation 1
-#' # summary(impacts_slm[[1]], zstats = TRUE, short = TRUE)
+#'  summary(impacts_slm[[1]], zstats = TRUE, short = TRUE)
 #' ## Impacts equation 2
-#' # summary(impacts_slm[[2]], zstats = TRUE, short = TRUE)
+#'  summary(impacts_slm[[2]], zstats = TRUE, short = TRUE)
 #' ## For SLX and SDEM models the output is a list of "WXImpact" objects
 #' ## See spatialreg::impacts for details.
 #' ## A SUR-SLX model
-#' # spcsur_slx <-spsurml(formula = Tformula, data = spc, 
-#' #                      type = "slx", listw = lwspc)
-#' # summary(spcsur_slx)
-#' # impacts_slx <- impactspsur(spcsur_slx, listw = lwspc)
-#' # summary(impacts_slx[[1]], zstats = TRUE, short = TRUE)
-#' # summary(impacts_slx[[2]], zstats = TRUE, short = TRUE)
+#'  spcsur_slx <-spsurml(formula = Tformula, data = spc, 
+#'                       type = "slx", listw = lwspc)
+#'  summary(spcsur_slx)
+#'  impacts_slx <- impactspsur(spcsur_slx, listw = lwspc)
+#'  summary(impacts_slx[[1]], zstats = TRUE, short = TRUE)
+#'  summary(impacts_slx[[2]], zstats = TRUE, short = TRUE)
 #' 
-#' # ## A SUR-SDM model
-#' # spcsur_sdm <-spsurml(formula = Tformula, data = spc, 
-#' #                      type = "sdm", listw = lwspc)
-#' # impacts_sdm <- impactspsur(spcsur_sdm, listw = lwspc, R = 1000)
-#' # summary(impacts_sdm[[1]], zstats = TRUE, short = TRUE)
-#' # summary(impacts_sdm[[2]], zstats = TRUE, short = TRUE)
-#' # # A SUR-SDM model with different spatial lags in each equation
-#' # TformulaD <- ~ UN83 + NMR83 + SMSA | UN80
-#' # spcsur_sdm2 <-spsurml(formula = Tformula, data = spc, type = "sdm", 
-#' #                       listw = lwspc, Durbin = TformulaD)
-#' # summary(spcsur_sdm2)                       
-#' # impacts_sdm2 <- impactspsur(spcsur_sdm2, listw = lwspc, R = 1000)
-#' # summary(impacts_sdm2[[1]], zstats = TRUE, short = TRUE)
-#' # summary(impacts_sdm2[[2]], zstats = TRUE, short = TRUE)
-#' # # A SUR-SLX model with different spatial lags in each equation
-#' # spcsur_slx2 <-spsurml(formula = Tformula, data = spc, 
-#' #                      type = "slx", listw = lwspc, Durbin = TformulaD)
-#' # summary(spcsur_slx2)
-#' # impacts_slx2 <- impactspsur(spcsur_slx2, listw = lwspc)
-#' # summary(impacts_slx2[[1]], zstats = TRUE, short = TRUE)
-#' # summary(impacts_slx2[[2]], zstats = TRUE, short = TRUE)
-
-#' # ## A SUR-SDEM model
-#' # spcsur_sdem <-spsurml(formula = Tformula, data = spc, 
-#' #                      type = "sdem", listw = lwspc)
-#' # impacts_sdem <- impactspsur(spcsur_sdem, listw = lwspc)
-#' # summary(impacts_sdem[[1]], zstats = TRUE, short = TRUE)
-#' # summary(impacts_sdem[[2]], zstats = TRUE, short = TRUE)
-#' #
-#' # ## A SUR-SARAR model
-#' # spcsur_sarar <-spsurml(formula = Tformula, data = spc, 
-#' #                      type = "sarar", listw = lwspc,
-#' #                      control = list(tol = 0.01))
-#' # impacts_sarar <- impactspsur(spcsur_sarar, listw = lwspc, R = 1000)
-#' # summary(impacts_sarar[[1]], zstats = TRUE, short = TRUE)
-#' # summary(impacts_sarar[[2]], zstats = TRUE, short = TRUE)
-#' # 
+#' ## A SUR-SDM model
+#'  spcsur_sdm <-spsurml(formula = Tformula, data = spc, 
+#'                       type = "sdm", listw = lwspc)
+#'  impacts_sdm <- impactspsur(spcsur_sdm, listw = lwspc, R = 1000)
+#'  summary(impacts_sdm[[1]], zstats = TRUE, short = TRUE)
+#'  summary(impacts_sdm[[2]], zstats = TRUE, short = TRUE)
+#' ## A SUR-SDM model with different spatial lags in each equation
+#'  TformulaD <- ~ UN83 + NMR83 + SMSA | UN80
+#'  spcsur_sdm2 <-spsurml(formula = Tformula, data = spc, type = "sdm", 
+#'                        listw = lwspc, Durbin = TformulaD)
+#'  summary(spcsur_sdm2)                       
+#'  impacts_sdm2 <- impactspsur(spcsur_sdm2, listw = lwspc, R = 1000)
+#'  summary(impacts_sdm2[[1]], zstats = TRUE, short = TRUE)
+#'  summary(impacts_sdm2[[2]], zstats = TRUE, short = TRUE)
+#'  ## A SUR-SLX model with different spatial lags in each equation
+#'  spcsur_slx2 <-spsurml(formula = Tformula, data = spc, 
+#'                       type = "slx", listw = lwspc, Durbin = TformulaD)
+#'  summary(spcsur_slx2)
+#'  impacts_slx2 <- impactspsur(spcsur_slx2, listw = lwspc)
+#'  summary(impacts_slx2[[1]], zstats = TRUE, short = TRUE)
+#'  summary(impacts_slx2[[2]], zstats = TRUE, short = TRUE)
+#' ### A SUR-SDEM model
+#'  spcsur_sdem <-spsurml(formula = Tformula, data = spc, 
+#'                       type = "sdem", listw = lwspc)
+#'  impacts_sdem <- impactspsur(spcsur_sdem, listw = lwspc)
+#'  summary(impacts_sdem[[1]], zstats = TRUE, short = TRUE)
+#'  summary(impacts_sdem[[2]], zstats = TRUE, short = TRUE)
+#' 
+#' ### A SUR-SARAR model
+#'  spcsur_sarar <-spsurml(formula = Tformula, data = spc, 
+#'                       type = "sarar", listw = lwspc,
+#'                       control = list(tol = 0.01))
+#'  impacts_sarar <- impactspsur(spcsur_sarar, listw = lwspc, R = 1000)
+#'  summary(impacts_sarar[[1]], zstats = TRUE, short = TRUE)
+#'  summary(impacts_sarar[[2]], zstats = TRUE, short = TRUE)
+#'  
+#' ## A SUR-GNM model
+#'  spcsur_gnm <-spsurml(formula = Tformula, data = spc, 
+#'                       type = "gnm", listw = lwspc,
+#'                       control = list(tol = 0.1))
+#'  impacts_gnm <- impactspsur(spcsur_gnm, listw = lwspc, R = 1000)
+#'  summary(impacts_gnm[[1]], zstats = TRUE, short = TRUE)
+#'  summary(impacts_gnm[[2]], zstats = TRUE, short = TRUE)
+#' ## A SUR-GNM model with different spatial lags in each equation
+#'  TformulaD <- ~ UN83 + NMR83 + SMSA | UN80
+#'  spcsur_gnm2 <-spsurml(formula = Tformula, data = spc, type = "gnm", 
+#'                        listw = lwspc, Durbin = TformulaD,
+#'                        control = list(tol = 0.1))
+#'  summary(spcsur_gnm2)                       
+#'  impacts_gnm2 <- impactspsur(spcsur_gnm2, listw = lwspc, R = 1000)
+#'  summary(impacts_gnm2[[1]], zstats = TRUE, short = TRUE)
+#'  summary(impacts_gnm2[[2]], zstats = TRUE, short = TRUE)
+#'  
 #' # ####################################
 #' # ######## G=1; Tm>1               ###
 #' # ####################################
 #' #
-#' # rm(list = ls()) # Clean memory
-#' # data(NCOVR, package="spsur")
-#' # nbncovr <- spdep::poly2nb(NCOVR.sf, queen = TRUE)
-#' # ## Some regions with no links...
-#' # lwncovr <- spdep::nb2listw(nbncovr, style = "W", zero.policy = TRUE)
-#' # Tformula <- HR80  | HR90 ~ PS80 + UE80 | PS90 + UE90
-#' # ## A SUR-SLM model
-#' # NCOVRSUR_slm <-spsurml(formula = Tformula, data = NCOVR.sf, 
-#' #                        type = "slm", listw = lwncovr, 
-#' #                        method = "Matrix", zero.policy = TRUE, 
-#' #                        control = list(fdHess = TRUE))
-#' # summary(NCOVRSUR_slm)
-#' # ## Use of trW to compute.
-#' # Wncovr <- as(spdep::listw2mat(lwncovr), "CsparseMatrix")
-#' # trwncovr <- spatialreg::trW(Wncovr, type = "MC")
-#' # impacts_NCOVRSUR_slm <- impactspsur(NCOVRSUR_slm, tr = trwncovr,
-#' #                                 R = 1000)
-#' # summary(impacts_NCOVRSUR_slm[[1]], zstats = TRUE, short = TRUE)
-#' # summary(impacts_NCOVRSUR_slm[[2]], zstats = TRUE, short = TRUE)
-#' # ## A SUR-SDM model
-#' # NCOVRSUR_sdm <-spsurml(formula = Tformula, data = NCOVR.sf, 
-#' #                        type = "sdm", listw = lwncovr, 
-#' #                        method = "Matrix", zero.policy = TRUE, 
-#' #                        control = list(fdHess = TRUE))
-#' # impacts_NCOVRSUR_sdm <- impactspsur(NCOVRSUR_sdm, tr = trwncovr,
-#' #                                 R = 1000)
-#' # summary(impacts_NCOVRSUR_sdm[[1]], zstats = TRUE, short = TRUE)
-#' # summary(impacts_NCOVRSUR_sdm[[2]], zstats = TRUE, short = TRUE)
+#'  rm(list = ls()) # Clean memory
+#'  data(NCOVR, package="spsur")
+#'  nbncovr <- spdep::poly2nb(NCOVR.sf, queen = TRUE)
+#' ### Some regions with no links...
+#'  lwncovr <- spdep::nb2listw(nbncovr, style = "W", zero.policy = TRUE)
+#'  Tformula <- HR80  | HR90 ~ PS80 + UE80 | PS90 + UE90
+#' ### A SUR-SLM model
+#'  NCOVRSUR_slm <-spsurml(formula = Tformula, data = NCOVR.sf, 
+#'                         type = "slm", listw = lwncovr, 
+#'                         method = "Matrix", zero.policy = TRUE, 
+#'                         control = list(fdHess = TRUE))
+#'  summary(NCOVRSUR_slm)
+#' ### Use of trW to compute.
+#'  Wncovr <- as(spdep::listw2mat(lwncovr), "CsparseMatrix")
+#'  trwncovr <- spatialreg::trW(Wncovr, type = "MC")
+#'  impacts_NCOVRSUR_slm <- impactspsur(NCOVRSUR_slm, tr = trwncovr,
+#'                                  R = 1000)
+#'  summary(impacts_NCOVRSUR_slm[[1]], zstats = TRUE, short = TRUE)
+#'  summary(impacts_NCOVRSUR_slm[[2]], zstats = TRUE, short = TRUE)
+#' ### A SUR-SDM model
+#'  NCOVRSUR_sdm <-spsurml(formula = Tformula, data = NCOVR.sf, 
+#'                         type = "sdm", listw = lwncovr, 
+#'                         method = "Matrix", zero.policy = TRUE, 
+#'                         control = list(fdHess = TRUE))
+#'  impacts_NCOVRSUR_sdm <- impactspsur(NCOVRSUR_sdm, tr = trwncovr,
+#'                                  R = 1000)
+#'  summary(impacts_NCOVRSUR_sdm[[1]], zstats = TRUE, short = TRUE)
+#'  summary(impacts_NCOVRSUR_sdm[[2]], zstats = TRUE, short = TRUE)
+#' }
 #' 
 #' @export
 impactspsur <- function(obj, ..., tr = NULL, 
@@ -299,7 +313,6 @@ impactspsur <- function(obj, ..., tr = NULL,
       Xi <- obj$X[idxres:(idxres + N - 1), names(coeffi)]
       icept <- grep("(Intercept)", colnames(Xi))
       iicept <- length(icept) > 0L
-      browser
       if (iicept) {
         lmi.model <- lm(formula(paste("y ~ ", 
                          paste(colnames(Xi)[c(-icept)], 
@@ -426,7 +439,9 @@ impactspsur <- function(obj, ..., tr = NULL,
     #return(res)
   } else {
     if (type == "slm") { type <- "lag" 
-    } else if (type == "sdm") { type <- "mixed" 
+    } else if (any(type == c("sdm", "gnm"))) { 
+      if (type == "gnm") deltas <- deltas[1:G] # supress lambda's in gnm
+      type <- "mixed" 
     } else if (type == "sarar") { type <- "sac" } 
     if (any(type == c("lag","mixed"))) {
       rho <- deltas
@@ -489,7 +504,7 @@ impactspsur <- function(obj, ..., tr = NULL,
         Pi <- cbind(b1i[1:(pi/2)], b1i[((pi/2) + 1):pi])
         bnamesi <- names(b1i[1:(pi/2)])
       }  
-      n <- N
+      n <- N*Tm
       mu_i <- NULL
       Sigma_i <- NULL
       if (!is.null(R)) {
